/*
This file is part of "Uakron Robot Project".

"Uakron Robot Project" is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

"Uakron Robot Project" is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "Rfid_Library.h"

#define MICRO_SECONDS_PER_MS 1000 

//how many ms should the timeout be for the 
//RFID reader. If it is too small, the 
//RFID chip cannot send a byte fast enough
//and the "no RFID" error will be returned.
#define ADJUSTABLE_TIMEOUT 30

//the total bytes recieved is 12
#define totalRecvBytes 12

//what pin controls /Enable on the RFID reader.
#define RFID_READER_ENABLE_PIN 0x02

/*
 * Initializes the communication between the RFID reader and
 * the AVR.
 */
void initRfidReader()
{
    //Set the baud. For 921600 hz and 2400 baud:
    //equation is:  UBRR = (freq / (2 * baud) ) - 1
    int baud = 23;
    
    UBRRH = (unsigned char)(baud >> 8);
    UBRRL = (unsigned char)(baud);

    //setup frame format:
    //        write to UCSRC    async      1 stop bit        8 data bits          no parity
    UCSRC =     (1<<URSEL)  | (0<<UMSEL) | (0<<USBS) | (1<<UCSZ0) | (1<<UCSZ1)| (0<<UPM0);
    
    //The enable is inverted logic. 0 means start transmission.

    DDRD = DDRD + RFID_READER_ENABLE_PIN;
    PORTD = PIND + RFID_READER_ENABLE_PIN;
}



//to startup the reciever
#define startRecv() UCSRB=(1<<RXEN);PORTD=PIND&(~RFID_READER_ENABLE_PIN)

//to stop the reciever
#define stopRecv() UCSRB=0;PORTD=PIND|RFID_READER_ENABLE_PIN

//to query if a byte is ready or not.
//returns 0 if no.
#define isByteReady() UCSRA&(1<<RXC)

//get the status, then the data byte from the reciever buffer.
#define getByte(data,status) status=UCSRA;data=UDR

/*
 * Reads 10 bytes of data from the RFID paddle. This
 * data is returned in the data buffer given to the method.
 * A status byte is returned with the method. If the
 * byte == NO_ERROR then all went well and the data 
 * buffer is valid. If any error occured, the data
 * buffer is not written to. The errors are defined
 * above.
 * @args data - The data buffer where the 10 bytes will
 *              be stored.
 * @return status of the transmission. See the errors
 * listed above for more details.
 */
byte readRfid(char data [10])
{
    
    int innerloop;
    int outerloop;
    int bytesRecvd = 0;
    byte status = 0;
    byte recvBuffer [12];
    
    byte dataBuffer,statusBuffer;

    //it is possible that there are left over bytes in the buffer.
    //there are a maximum of 2 possible.
    getByte(dataBuffer,statusBuffer);
    getByte(dataBuffer,statusBuffer);
    
    startRecv();

    //enter a loop. if the loop is exited before all the bytes are
    //collected, then a timeout has occured
    for(outerloop = MICRO_SECONDS_PER_MS; outerloop != 0; outerloop--)
    {
        for(innerloop = ADJUSTABLE_TIMEOUT; innerloop != 0; innerloop--)
        {
            if(isByteReady())
            {
                getByte(dataBuffer,statusBuffer);

                recvBuffer[bytesRecvd] = dataBuffer;
                    
                bytesRecvd++;

                //(re)set timeout counters
                if(bytesRecvd == 12)
                {
                   goto RECV_DONE;
                }
                else
                {
                    innerloop = MICRO_SECONDS_PER_MS;
                    outerloop = ADJUSTABLE_TIMEOUT;
                }

            }
                
        }
    }

    RECV_DONE:

    stopRecv();

    if(bytesRecvd == 0)
    {
        status = NO_RFID_ERROR;
    }
    else if(bytesRecvd < 12)
    {
        status = RFID_READ_TIMEOUT_ERROR;
    }
    else
    {
        status = NO_ERROR;
            
        for(innerloop = 10; innerloop > 0; innerloop--)
        {
            data[innerloop-1] = recvBuffer[innerloop];
        }

    }

    return status;
}


